<?php

namespace AppBundle\Api;

use Monolog\Logger;

/**
 * Class CsServerSettings
 * @package AppBundle\Api
 */
class CsServerSettings{

    const SERVICE_CS_DAEMON = 0;
    const SERVICE_DS_DAEMON = 1;
    const SERVICE_ROADWARRIOR = 2;

    const STATUS_OFFLINE = 0;
    const STATUS_ONLINE = 1;

    const ACTION_START = "start";
    const ACTION_STOP = "stop";
    const ACTION_RESTART = "restart";

    /**
     * @var Logger
     */
    protected $log;

    /**
     * @var \PhpClientApi
     */
    protected $phpClientApi;

    /**
     * @var \PhpSettingsApi
     */
    protected $phpSettingsApi;

    /**
     * CsServerSettings constructor.
     * @param $phpClientApi
     * @param $phpSettingsApi
     * @param Logger $log
     */
    function __construct($phpClientApi, $phpSettingsApi, Logger $log) {

        $this->phpClientApi = $phpClientApi;
        $this->phpSettingsApi = $phpSettingsApi;
        $this->log = $log;

    }

    /**
     * @return array
     */
    public function getCsStats(){

        try{

            return $this->phpClientApi->get_cs_stats();

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            return array();

        }

    }

    /**
     * @return array
     */
    public function getSettingsSectionsInfo(){

        try{

            return $this->phpSettingsApi->get_sections_info();

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            return array();

        }

    }

    /**
     * @param $sectionName
     * @return mixed
     */
    public function getSettingsBySection($sectionID){

        try{

            return $this->phpSettingsApi->get_section($sectionID);

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            return array();

        }


    }

    /**
     * @param array $settings
     * @return bool
     */
    public function updateSettings(array $settings){


        try{

            $this->phpSettingsApi->update_settings($settings);
	    return "";
            //return true;

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);
	    return $e->getMessage();
            //return false;

        }

    }

    /**
     * @return bool
     */
    public function isLicenseValid(){

        try{

            //return $this->phpClientApi->get_cs_online_status();
            return $this->phpClientApi->is_cs_license_valid();

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;

        }

    }

    /**
     * @param $service
     * @return bool
     */
    public function getServiceStatus($service){

        try{

            return $this->phpClientApi->get_service_status($service);

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;

        }

    }

    /**
     * @param $service
     * @param $action
     * @return mixed
     * @throws \Exception
     */
    public function manageService($service, $action){

        try{

            return $this->phpClientApi->manage_service($service, $action);

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            throw new \Exception($e->getMessage());

        }

    }

    /**
     * @return bool
     */
    public function getCsVersionString(){

        try{

            return $this->phpClientApi->get_cs_version_string();

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;

        }

    }

    /**
     * @param $file
     * @return bool|string
     */
    public function uploadFile($file){

        try{

            $this->phpClientApi->vpn_update_tarball($file);
            return "";
            //return true;

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            return $e->getMessage();
            //return false;

        }


    }

    /**
     * @return array
     */
    public function getLicenseInfo(){

        try{

            return $this->phpClientApi->get_license_info();

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            return array();

        }

    }

    /**
     * @return string
     */
    public function getCSSysLog(){
        try{
            return $this->phpClientApi->get_cs_system_log(1000);
        }catch (\Exception $e){
            $this->logError(__FUNCTION__, func_get_args(), $e);
            return $e;
        }
    }

    /**
     * @return string
     */
    public function getOpenVPNLog(){
        try{
            return $this->phpClientApi->get_cs_ovpn_log(1000);
        }catch (\Exception $e){
            $this->logError(__FUNCTION__, func_get_args(), $e);
            return $e;
        }
    }

    /**
     * @return array
     */
    public function getConnectionLog(){
        $where = array();
        $orderBy = array(array("conn_event_time",  "desc"));

        try{
            $connlogs = $this->phpClientApi->get_all_connlogs(0, 1000, $where, $orderBy); // params: offset, limit, where_cond, order_by
            if($connlogs == null){
                $connlogsReadable = 'There are no connection logs.';
            } else {
                $connlogsReadable = $this->connlogsReadable(array_reverse($connlogs));
            }
            return $connlogsReadable;
        }catch (\Exception $e){
            $this->logError(__FUNCTION__, func_get_args(), $e);
            return $e;
        }
    }

    /**
     * @param $deviceId
     * @return array
     */
    public function getConnectionLogForDevice($deviceId){

        $where = array();
        $orderBy = array(array("conn_event_time",  "desc"));

        try{
            $connlogs = $this->phpClientApi->get_connlogs_by_id($deviceId, 0, 200, $where, $orderBy); // params: device-id, offset, limit, where_cond, order_by );
            if($connlogs == null){
                $connlogsReadable = 'There are no connection logs for this device.';
            } else {
                $connlogsReadable = $this->connlogsReadable(array_reverse($connlogs));
            }
            return $connlogsReadable;
        }catch (\Exception $e){
            $this->logError(__FUNCTION__, func_get_args(), $e);
            return $e;
        }
    }

    /**
     * @param $fileName
     * @param bool $deleteFile
     * @return mixed
     * @throws \Exception
     */
    public function addLicence($fileName, $deleteFile = false){

        try{

            return $this->phpClientApi->add_license($fileName, $deleteFile);

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            throw $e;

        }

    }

    /**
     * @param $functionName
     * @param array $arguments
     * @param \Exception|null $exception
     */
    private function logError($functionName, array $arguments, \Exception $exception = null){

        try{

            $ref = new \ReflectionMethod($this, $functionName);
            $functionParameters = [];

            foreach($ref->getParameters() as $key => $currentParameter) {

                $functionParameters[$currentParameter->getName()] = $arguments[$key];

            }

            $this->log->addError('Exception ' . (($exception !== null) ? '"' .$exception->getMessage(). '"' : '').' on '. $functionName .', parameters:' . print_r(serialize($functionParameters), true));

        }catch (\Exception $e){

            $this->log->addError('Exception ' . (($exception !== null) ? '"' .$exception->getMessage(). '"' : '').' on '. $functionName);

        }

    }

    /**
     * @param $connlogs
     *
     * @return string
     */
    private function connlogsReadable($connlogs){
        $connlogsall = null;
        $eventType = null;
        foreach ($connlogs as $cnlg){

            // bytes to fitting units
            $cnlg["bytes_sent"] = $this->formatBytes($cnlg["bytes_sent"]);
            $cnlg["bytes_received"] = $this->formatBytes($cnlg["bytes_received"]);

            // uptime to human format
            $s = (int)$cnlg["uptime"];
            $cnlg["uptime"] = sprintf('%dd %02dh:%02dm', $s/86400, $s/3600%24, $s/60%60);

            // log sentence
            if($cnlg["conn_event_type"] == 0){
                $eventType = " went Offline, ";
                $cnlg = [date("D M d H:i:s Y", $cnlg["conn_event_time"]), " Device #", $cnlg["device_id"], $eventType, $cnlg["bytes_sent"], " sent, " , $cnlg["bytes_received"], " received, " , $cnlg["uptime"], " uptime"];
            } elseif ($cnlg["conn_event_type"] == 1){
                $eventType = " went Online";
                $cnlg = [date("D M d H:i:s Y", $cnlg["conn_event_time"]), " Device #", $cnlg["device_id"], $eventType ];
            }
            $cnlgstring = implode("", $cnlg);
            $connlogsall .= $cnlgstring . "\n";
        }
        return $connlogsall;
    }

    /**
     * @param $size
     *
     * @return string
     */
    private function formatBytes($size){
        $base = log(floatval($size), 1024);
        $suffixes = array('B', 'kB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB');
        $precision = array('0', '0', '2', '3', '3', '3', '3', '3');
        return round(pow(1024, $base - floor($base)), $precision[floor($base)]) .' '. $suffixes[floor($base)];
    }

}
