<?php


namespace AppBundle\Controller;

use AppBundle\Api\CsServerSettings;
use AppBundle\Api\FwRules;
use AppBundle\Entity\DeviceFwRule;
use AppBundle\Entity\DeviceFwRules;
use AppBundle\Entity\DeviceLan;
use AppBundle\Entity\DeviceLans;
use AppBundle\Entity\User;
use AppBundle\Form\DeviceGeneral;
use AppBundle\Form\DeviceNetworks;
use AppBundle\Form\DeviceProxy;
use Doctrine\Common\Collections\ArrayCollection;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Config\Definition\Exception\Exception;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use AppBundle\Form\Device;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * Class DevicesController
 * @package AppBundle\Controller
 */
class DevicesController extends BaseController
{

    /**
     * @var \AppBundle\Entity\Device
     */
    private $device;

    /**
     * @Route("/routers", name="app_manage_devices")
     *
     * @param Request $request Request
     *
     * @return Response
     */
    public function manageDevicesAction(Request $request){

        //list($start, $length, $aOrder, $filter) = $this->parseData($request);
        /*
        /** @var \AppBundle\Api\Device $apiDevice */
        /*
        $apiDevice = $this->container->get('cs_api_device');
        $routers = $apiDevice->getAllRouters(0,3);

          */
            //$start, $length, $aOrder, $filter);

        //$modRouters = array();

        /*
        foreach ($routers as &$router){
            if($router["disabled"] == "1"){
                $router["online"] = "2";
            }
        }

        var_dump($routers);
        */
        //var_dump($modRouters);


        return $this->render('app/pages/device/manageDevices.html.twig', []);

    }

    /**
     * @Route("/devices-validation", name="app_manage_devices_validation")
     *
     * @param Request $request Request
     *
     * @return Response
     */
    public function manageDevicesValidationAction(Request $request){

        return $this->render('app/pages/device/manageDevicesValidation.html.twig', []);



    }

    /**
     * @Route("device/edit/{id}", options = { "expose" = true }, name="app_edit_device")
     *
     * @param Request $request Request
     *
     * @return Response
     */
    public function editDeviceAction(Request $request, $id)
    {

        $this->denyAccessUnlessGranted(User::ROLE_DEVICE_ADMIN);

        //API
        /** @var \AppBundle\Api\Device $apiDevice */
        $apiDevice = $this->container->get('cs_api_device');
        /** @var FwRules $apiFwRules */
        $apiFwRules = $this->container->get('cs_api_fw_rules');
        /** @var CsServerSettings $CsServerSettings */
        $apiCsServerSettings = $this->container->get('cs_api_server_settings');
        /** @var \AppBundle\Api\StandaloneVpnClient $apiStandaloneVpnClient */
        $apiStandaloneVpnClient = $this->container->get('cs_api_standalone_vpn_client');

        $incomingDefaultPolicy = $apiFwRules->getDeviceFwPolicy($id, FwRules::TRAFFIC_DIRECTION_INCOMING);

        $outgoingDefaultPolicy = $apiFwRules->getDeviceFwPolicy($id, FwRules::TRAFFIC_DIRECTION_OUTGOING);

        //Actual Incoming FW Rules
        $where_condition['related_policy'] = FwRules::RELATED_POLICY_DROP; // ACCEPT=0, DROP=1
        $where_condition['traffic_direction'] = FwRules::TRAFFIC_DIRECTION_INCOMING; // INCOMING=1, OUTGOING=2

        $fwRules = $apiFwRules->getDeviceFwRules($id, $where_condition);


        $incomingAcceptFwRules = $this->setFwRules($id, $fwRules);

        //Incoming FW Rules
        $where_condition['related_policy'] = FwRules::RELATED_POLICY_ACCEPT; // ACCEPT=0, DROP=1
        $where_condition['traffic_direction'] = FwRules::TRAFFIC_DIRECTION_INCOMING; // INCOMING=1, OUTGOING=2

        $fwRules = $apiFwRules->getDeviceFwRules($id, $where_condition);


        $incomingDropFwRules = $this->setFwRules($id, $fwRules);

        $where_condition['related_policy'] = FwRules::RELATED_POLICY_DROP; // ACCEPT=0, DROP=1
        $where_condition['traffic_direction'] = FwRules::TRAFFIC_DIRECTION_OUTGOING; // INCOMING=1, OUTGOING=2

        $fwRules = $apiFwRules->getDeviceFwRules($id, $where_condition);


        $outgoingAcceptFwRules = $this->setFwRules($id, $fwRules);

        $where_condition['related_policy'] = FwRules::RELATED_POLICY_ACCEPT; // ACCEPT=0, DROP=1
        $where_condition['traffic_direction'] = FwRules::TRAFFIC_DIRECTION_OUTGOING; // INCOMING=1, OUTGOING=2

        $fwRules = $apiFwRules->getDeviceFwRules($id, $where_condition);


        $outgoingDropFwRules = $this->setFwRules($id, $fwRules);

        $authorizationChecker = $this->container->get('security.authorization_checker');

        //device information
        $deviceDetail = $apiDevice->getDeviceById($id);
        $deviceConfig = $apiDevice->getDeviceConfigById($id);
        $deviceConnlog = $apiCsServerSettings->getConnectionLogForDevice($id);
        if($deviceDetail['device_type'] == "2") {
            $routerDetail = $apiDevice->getRouterById($id);
            $aLans = $apiDevice->getLans($id);
            $internetAccess = $apiDevice->getInternetAccess($id);
        } elseif($deviceDetail['device_type'] == "1") {
            $standaDetail = $apiStandaloneVpnClient->getStandaloneVpnClientById($id);
            $standaDetail['country'] = $this->getCountry($standaDetail['country']); // get country name from code
            if(time() > $standaDetail['expiration_date']){
                $standaDetail['red'] = 'color:#bb4a1e';
            } else {
                $standaDetail['red'] = '';
            }
        }

        list($w, $x, $y) = explode('.', $deviceDetail['ovpn_ip']);
        $virtualIpBase = $w . '.' . $x . '.' . $y;
        $virtualMaskBase = '255.255.255';

        if (!$deviceDetail) {

            $this->addFlash(
                'error',
                'Error while reading device detail!'
            );

            return $this->redirectToRoute('app_manage_devices');

        }

            $lans = $this->container->get('device_lans');

            if(!empty($aLans)){

                foreach ($aLans as $lan) {

                    $deviceLan = clone $this->container->get('device_lan');
                    $deviceLan->setDeviceId($id);
                    $deviceLan->setName($lan['lan_iface']);
                    $deviceLan->setMode($lan['mode']);
                    $deviceLan->setIfaceIp($lan['lan_iface_ip']);
                    $deviceLan->setIp($lan['lan_ip']);
                    $deviceLan->setNetmask($lan['lan_netmask']);
                    $deviceLan->setDhcpEnabled(($lan['dhcp_enabled'] == '1') ? true : false);
                    $deviceLan->setDhcpFrom($lan['dhcp_from']);
                    $deviceLan->setDhcpTo($lan['dhcp_to']);

                    $ip = '';
                    if($lan['virtual_ip'] != ''){

                        $array = explode('.', $lan['virtual_ip']);
                        $ip = $array[3];

                    }

                    $deviceLan->setVirtualIp($ip);

                    $maskIp = '';
                    if($lan['virtual_mask'] != ''){

                        $array = explode('.', $lan['virtual_mask']);
                        $maskIp = $array[3];

                    }

                    $deviceLan->setVirtualNetmask($maskIp);
                    $deviceLan->setErrorCode((isset($lan['error_code']) ? $lan['error_code'] : null));
                    $lans->getLans()->add($deviceLan);

                }

            }

        /** @var Device $device */
        $this->device = $this->container->get('device');
        $this->device->setId($id);
        $this->device->setName($deviceDetail['name']);
        $this->device->setLans($lans->getLans());
        $this->device->setDefaultFwIncomingPolicy($incomingDefaultPolicy);
        $this->device->setDefaultFwOutgoingPolicy($outgoingDefaultPolicy);
        $this->device->setIncomingAcceptFwRules($incomingAcceptFwRules->getFwRules());
        $this->device->setIncomingDropFwRules($incomingDropFwRules->getFwRules());
        $this->device->setOutgoingAcceptFwRules($outgoingAcceptFwRules->getFwRules());
        $this->device->setOutgoingDropFwRules($outgoingDropFwRules->getFwRules());
        $this->device->setValidated($deviceDetail['validated']);

        $stats = $apiDevice->getDeviceStats($id);

        $aNetworks = array();

        $networks = $apiDevice->getNetworksByDeviceId($id);

        if (!empty($networks)) {

            foreach ($networks as $network) {

                $aNetworks[$network['name']] = $network['network_id'];

            }

        }

        $this->device->setNetworks($aNetworks);

        $deviceDb = clone ($this->device);

        $fixedPartUrl = $deviceDetail['proxy_url'];
        $ovpn_ip_mask = str_replace(".", "-", $deviceDetail['ovpn_ip']) . '-';
        //get rest after last "-"
        $fixedPartUrl = str_replace($ovpn_ip_mask, '' , $fixedPartUrl);
        //get string until first "."
        $fixedPartUrl = explode(".", $fixedPartUrl, 2);
        $tmp_r = $fixedPartUrl;
        unset($tmp_r[0]);
        $fixedPartUrl = $fixedPartUrl[0];
        $domainName = implode('.', $tmp_r);

        /*
        $generalForm = $this->createForm(DeviceGeneral::class, $this->device, array(
            'apiDevice' => $apiDevice,
            'validated_disabled' => $this->device->getValidated(),
            'authorizationChecker' => $authorizationChecker));

        $generalForm->handleRequest($request);

        if ($generalForm->isSubmitted() && $generalForm->isValid()) {
            if ($authorizationChecker->isGranted('ROLE_DEVICE_ADMIN')) {
                $name = $this->device->getName();
                try {
                    $apiDevice->setDeviceName($this->device->getId(), $name);
                } catch (\Exception $e) {
                    $errorMessage = "Name of device could not be changed.";  //$e->getMessage();
                }
            }
            if('apply' === $generalForm->getClickedButton()->getName()) {
                if(isset($errorMessage)){
                    $this->addFlash(
                        'error',
                        $errorMessage . ' '
                    );
                }else{

                    $this->addFlash(
                        'notice',
                        'Your changes were saved!'
                    );
                }
                return $this->redirect($request->getUri());
            }else{
                if(isset($errorMessage)){
                    $this->addFlash(
                        'error',
                        $errorMessage . ' '
                    );
                }else{
                    $this->addFlash(
                        'notice',
                        'Your changes were saved!'
                    );
                }
                if($deviceDetail['device_type'] == "2") {
                    return $this->redirectToRoute('app_manage_devices');
                } elseif($deviceDetail['device_type'] == "1") {
                    return $this->redirectToRoute('app_standalone_vpn_clients');
                } else {
                    return $this->redirectToRoute('app_dashboard');
                }
            }
        }
        */

        // LANs form
        $form = $this->createForm(Device::class, $this->device, array(
            'apiDevice' => $apiDevice, 'validated_disabled' => $this->device->getValidated(),
            'deviceId' => $id, 'virtualIpBase' => $virtualIpBase, 'virtualMaskBase' => $virtualMaskBase));

        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {

            $lan_errors = $apiDevice->setLans($this->device->getData()['lans'], $id);

            /*
            if ($authorizationChecker->isGranted('ROLE_NET_MANAGER')){

                $networksToJoin = array_diff($this->device->getNetworks(), $deviceDb->getNetworks());

                foreach ($networksToJoin as $network){

                    $apiDevice->joinDeviceNetwork($this->device->getId(), $network);

                }

                $networksToLeave = array_diff($deviceDb->getNetworks(), $this->device->getNetworks());

                foreach ($networksToLeave as $network){

                    $apiDevice->leaveDeviceNetwork($this->device->getId(), $network);

                }

            }
            */

            if (isset($lan_errors)){

                $error_text = json_encode($lan_errors); // Conver array into something that can be printed out
                $this->addFlash(
                    'error',
                    substr($error_text, 1, strlen($error_text)-2) // Trim first and last character
                );

            }else{

                $this->addFlash(
                    'notice',
                    'Your changes were saved.'
                );

            }

            if('apply' === $form->getClickedButton()->getName()) {

                return $this->redirect($request->getUri());

            }else{ //save button

                if(isset($_SESSION['_deviceReturnPath'])){
                    //return $this->redirectToRoute($_SESSION['_deviceReturnPath']);
                    return $this->redirect($_SESSION['_deviceReturnPath']);
                } else {
                    return $this->redirectToRoute('app_manage_devices');
                }



            }

        }

        $networksForm = $this->createForm(DeviceNetworks::class, $this->device, array(
            'apiDevice' => $apiDevice, 'validated_disabled' => $this->device->getValidated(),
            'authorizationChecker' => $authorizationChecker));

        $networksForm->handleRequest($request);

        if ($networksForm->isSubmitted() && $networksForm->isValid()) {


            if ($authorizationChecker->isGranted('ROLE_NET_MANAGER')){

                $networksToJoin = array_diff($this->device->getNetworks(), $deviceDb->getNetworks());

                foreach ($networksToJoin as $network){

                    $apiDevice->joinDeviceNetwork($this->device->getId(), $network);

                }

                $networksToLeave = array_diff($deviceDb->getNetworks(), $this->device->getNetworks());

                foreach ($networksToLeave as $network){

                    $apiDevice->leaveDeviceNetwork($this->device->getId(), $network);

                }

                $this->addFlash(
                    'notice',
                    'Your changes were saved.'
                );

            }

            if('apply' === $networksForm->getClickedButton()->getName()) {

                return $this->redirect($request->getUri());

            }else{//save button

                if(isset($_SESSION['_deviceReturnPath'])){
                    return $this->redirect($_SESSION['_deviceReturnPath']);
                } else {
                    return $this->redirectToRoute('app_manage_devices');
                }

            }

        }

        /*
        $allNetworks = $apiDevice->getAllNetworks(0,1000);

        $aAllNetworks = array();

        if($allNetworks){

            foreach ($allNetworks as $network){

                $aAllNetworks[$network['network_id']] = $network['name'];

            }

        }
        */

        $proxyForm = $this->createForm(DeviceProxy::class, $this->device, array('apiDevice' => $apiDevice,
            'validated_disabled' => $this->device->getValidated()));

        $proxyForm->handleRequest($request);

        if ($proxyForm->isSubmitted() && $proxyForm->isValid()) {

            // is checkbox checked or not?
                $deviceProxy = (($this->device->getDeviceProxy() === true) ? 1 : 0);
                $lanProxy = (($this->device->getLanProxy() === true) ? 1 : 0);

                try{
                    $apiDevice->setDeviceProxy($this->device->getId(), $deviceProxy);
                } catch (\Exception $e) {

                    $errorMessage = $e->getMessage();

                }

            try{
                $apiDevice->setLanProxy($this->device->getId(), $lanProxy);
            } catch (\Exception $e) {

                $errorMessage2 = $e->getMessage();

            }

            if('apply' === $proxyForm->getClickedButton()->getName()) {

                if(isset($errorMessage)){

                    $this->addFlash(
                        'error',
                        $errorMessage . ' '
                    );
                }elseif (isset($errorMessage2)){

                    $this->addFlash(
                        'error',
                        $errorMessage2 . ' '
                    );
                }else{

                    $this->addFlash(
                        'notice',
                        'Your changes were saved!'
                    );

                }

                return $this->redirect($request->getUri());

            }else{ //save button

                if(isset($errorMessage)){

                    $this->addFlash(
                        'error',
                        $errorMessage . ' '
                    );
                }elseif (isset($errorMessage2)){

                    $this->addFlash(
                        'error',
                        $errorMessage2 . ' '
                    );
                }else{

                    $this->addFlash(
                        'notice',
                        'Your changes were saved!'
                    );

                }

                if(isset($_SESSION['_deviceReturnPath'])){
                    return $this->redirect($_SESSION['_deviceReturnPath']);
                } else {
                    return $this->redirectToRoute('app_manage_devices');
                }

            }

        }

        //$object->setCheckboxValue(true);

        //var_dump($this->device->getIncomingDropFwRules());




        $fwRulesForm = $this->createForm(\AppBundle\Form\DeviceFwRules::class, $this->device);

        $fwRulesForm->handleRequest($request);

        if ($fwRulesForm->isSubmitted() && $fwRulesForm->isValid()) {

            try{

                $this->setChangedFwRules($deviceDb, $this->device);

            }catch (\Exception $e){

                $errorMessage = $e->getMessage();

            }

            if('apply' === $fwRulesForm->getClickedButton()->getName()) {

                if(isset($errorMessage)){

                $this->addFlash(
                    'fwError',
                    $errorMessage . ', no FW rule was set.'
                );
                }else{

                    $this->addFlash(
                        'notice',
                        'Your changes were saved!'
                    );

                }

                return $this->redirect($request->getUri());

            }else{//save button

                if(isset($errorMessage)){

                    $this->addFlash(
                        'error',
                        $errorMessage . ', no FW rule was set.'
                    );

                }else{

                    $this->addFlash(
                        'notice',
                        'Your changes were saved!'
                    );

                }

                if(isset($_SESSION['_deviceReturnPath'])){
                    return $this->redirect($_SESSION['_deviceReturnPath']);
                } else {
                    return $this->redirectToRoute('app_manage_devices');
                }

            }

        }




        if($deviceDetail['device_type'] == "2") { //router
            return $this->render('app/pages/device/deviceEdit.html.twig', [
                //'generalForm' => $generalForm->createView(),
                'form' => $form->createView(),
                'networksForm' => $networksForm->createView(),
                'proxyForm' => $proxyForm->createView(),
                'fwRulesForm' => $fwRulesForm->createView(),
                'device' => $this->device,
                'deviceDetail' => $deviceDetail,
                'routerDetail' => $routerDetail,
                'deviceConfig' => $deviceConfig,
                'lans' => $aLans,
                'stats' => $stats,
                'virtualIpBase' => $virtualIpBase,
                'virtualMaskBase' => $virtualMaskBase,
                'fixedPartUrl' => $fixedPartUrl,
                'domainName' => $domainName,
                'deviceConnlog' => $deviceConnlog,
                'internetAccess' => $internetAccess,
            ]);
        } elseif($deviceDetail['device_type'] == "1") { //standa
            return $this->render('app/pages/device/deviceEdit.html.twig', [
                //'generalForm' => $generalForm->createView(),
                'form' => $form->createView(),
                'networksForm' => $networksForm->createView(),
                'proxyForm' => $proxyForm->createView(),
                'fwRulesForm' => $fwRulesForm->createView(),
                'device' => $this->device,
                'deviceDetail' => $deviceDetail,
                'standaDetail' => $standaDetail,
                'deviceConfig' => $deviceConfig,
                'stats' => $stats,
                'virtualIpBase' => $virtualIpBase,
                'virtualMaskBase' => $virtualMaskBase,
                'fixedPartUrl' => $fixedPartUrl,
                'domainName' => $domainName,
                'deviceConnlog' => $deviceConnlog,
            ]);
        }
    }
/*
        return $this->render('app/pages/device/deviceEdit.html.twig', [
            'generalForm' => $generalForm->createView(),
            'form' => $form->createView(),
            'networksForm' => $networksForm->createView(),
            'proxyForm' => $proxyForm->createView(),
            'fwRulesForm' => $fwRulesForm->createView(),
            'device' => $this->device,
            'deviceDetail' => $deviceDetail,
            'routerDetail' => $routerDetail,
            'standaDetail' => $standaDetail,
            'deviceConfig' => $deviceConfig,
            'lans' => $aLans,
            'stats' => $stats,
            //'networks' => $aAllNetworks,
            'virtualIpBase' => $virtualIpBase,
            'virtualMaskBase' => $virtualMaskBase,
            'fixedPartUrl' => $fixedPartUrl,
            'deviceConnlog' => $deviceConnlog
        ]);
    }
*/

    /**
     * @param array $array
     * @param $key
     * @param $value
     * @return array
     */
    public function searchSubArray(Array $array, $key, $value){

        foreach ($array as $k => $subarray){

            if (isset($subarray[$key]) && $subarray[$key] == $value)

                return array($k => $subarray);

        }

    }

    /**
     * @param $deviceId
     * @param array $apiFwRules
     * @return DeviceFwRules|object
     */
    private function setFwRules($deviceId, array $apiFwRules){

        $fwRules = clone $this->container->get('device_fw_rules');

        foreach ($apiFwRules as $apiFwRule){

            $fwRule = clone $this->container->get('device_fw_rule');
            $fwRule->setDeviceId($deviceId);
            $fwRule->setEnabled(($apiFwRule['enabled'] == '1') ? true : false);
            $fwRule->setTrafficDirection($apiFwRule['traffic_direction']);
            $fwRule->setProtocol($apiFwRule['protocol']);
            $fwRule->setSourceIps($apiFwRule['src_ips']);
            $fwRule->setDestinationIps($apiFwRule['dst_ips']);
            $fwRule->setPorts($apiFwRule['ports']);
            $fwRules->setFwRule($fwRule);

        }

        return $fwRules;

    }


    /**
     * @param \AppBundle\Entity\Device $objectOriginal
     * @param \AppBundle\Entity\Device $updatedObject
     */
    private function setChangedFwRules(\AppBundle\Entity\Device $objectOriginal, \AppBundle\Entity\Device $updatedObject){


        $apiFwRules = $this->container->get('cs_api_fw_rules');

        if($objectOriginal->getDefaultFwIncomingPolicy() != $updatedObject->getDefaultFwIncomingPolicy()){

            $apiFwRules->setDeviceFwPolicy($updatedObject->getId(), FwRules::TRAFFIC_DIRECTION_INCOMING, $updatedObject->getDefaultFwIncomingPolicy());

        }

        if($objectOriginal->getDefaultFwOutgoingPolicy() != $updatedObject->getDefaultFwOutgoingPolicy()){

            $apiFwRules->setDeviceFwPolicy($updatedObject->getId(), FwRules::TRAFFIC_DIRECTION_OUTGOING, $updatedObject->getDefaultFwOutgoingPolicy());

        }

        $fwRulesArray = array();

        if($objectOriginal->getIncomingAcceptFwRules() != $updatedObject->getIncomingAcceptFwRules()){

            $data = $updatedObject->getIncomingAcceptFwRules()->toArray();

            $array = $this->convertToArray($data, FwRules::TRAFFIC_DIRECTION_INCOMING, FwRules::RELATED_POLICY_DROP);

            $fwRulesArray = array_merge($fwRulesArray, $array);

        }else{

            $fwRulesArray = array_merge($fwRulesArray, $this->convertToArray($objectOriginal->getIncomingAcceptFwRules()->toArray(),
                FwRules::TRAFFIC_DIRECTION_INCOMING, FwRules::RELATED_POLICY_DROP));

        }

        if($objectOriginal->getIncomingDropFwRules() != $updatedObject->getIncomingDropFwRules()){

            $data = $updatedObject->getIncomingDropFwRules()->toArray();

            $array = $this->convertToArray($data, FwRules::TRAFFIC_DIRECTION_INCOMING, FwRules::RELATED_POLICY_ACCEPT);

            $fwRulesArray = array_merge($fwRulesArray, $array);

        }else{

            $fwRulesArray = array_merge($fwRulesArray, $this->convertToArray($objectOriginal->getIncomingDropFwRules()->toArray(),
                FwRules::TRAFFIC_DIRECTION_INCOMING, FwRules::RELATED_POLICY_ACCEPT));

        }

        if($objectOriginal->getOutgoingAcceptFwRules() != $updatedObject->getOutgoingAcceptFwRules()){

            $data = $updatedObject->getOutgoingAcceptFwRules()->toArray();

            $array = $this->convertToArray($data, FwRules::TRAFFIC_DIRECTION_OUTGOING, FwRules::RELATED_POLICY_DROP);

            $fwRulesArray = array_merge($fwRulesArray, $array);

        }else{

            $fwRulesArray = array_merge($fwRulesArray, $this->convertToArray($objectOriginal->getOutgoingAcceptFwRules()->toArray(),
                FwRules::TRAFFIC_DIRECTION_OUTGOING, FwRules::RELATED_POLICY_DROP));

        }

        if($objectOriginal->getOutgoingDropFwRules() != $updatedObject->getOutgoingDropFwRules()){

            $data = $updatedObject->getOutgoingDropFwRules()->toArray();

            $array = $this->convertToArray($data, FwRules::TRAFFIC_DIRECTION_OUTGOING, FwRules::RELATED_POLICY_ACCEPT);

            $fwRulesArray = array_merge($fwRulesArray, $array);

        }else{

            $fwRulesArray = array_merge($fwRulesArray, $this->convertToArray($objectOriginal->getOutgoingDropFwRules()->toArray(),
                FwRules::TRAFFIC_DIRECTION_OUTGOING, FwRules::RELATED_POLICY_ACCEPT));


        }

        if(!empty($fwRulesArray)){

            $apiFwRules->addDeviceFwRules($updatedObject->getId(), $fwRulesArray);

        }else{

            $apiFwRules->addDeviceFwRules($updatedObject->getId(), array());

        }

    }

    /**
     * @param $data
     * @param $direction
     * @param $policy
     * @return array
     */
    private function convertToArray($data, $direction, $policy){

        $items = array();
        $item = array();

        foreach ($data as $dataItem){
            $item['enabled'] = ($dataItem->getEnabled() == true) ? '1' : '0';
            $item['traffic_direction'] = (string) $direction;
            $item['related_policy'] = (string) $policy;
            $item['protocol'] = $dataItem->getProtocol();
            $item['src_ips'] = ($dataItem->getSourceIps() == null) ? '' : $dataItem->getSourceIps();
            $item['dst_ips'] = ($dataItem->getDestinationIps() == null) ? '' : $dataItem->getDestinationIps();
            $item['ports'] = ($dataItem->getPorts() == null) ? '' : $dataItem->getPorts();

            $items[] = $item;
        }

        return $items;

    }

    /**
     * @param $countryCode
     * @return string
     */
    private function getCountry($countryCode){

        switch($countryCode)
        {
            case 'AD': return 'Andorra'; break;
            case 'AE': return 'United Arab Emirates'; break;
            case 'AF': return 'Afghanistan'; break;
            case 'AG': return 'Antigua and Barbuda'; break;
            case 'AI': return 'Anguilla'; break;
            case 'AL': return 'Albania'; break;
            case 'AM': return 'Armenia'; break;
            case 'AN': return 'Netherlands Antilles'; break;
            case 'AO': return 'Angola'; break;
            case 'AQ': return 'Antarctica'; break;
            case 'AR': return 'Argentina'; break;
            case 'AS': return 'American Samoa'; break;
            case 'AT': return 'Austria'; break;
            case 'AU': return 'Australia'; break;
            case 'AW': return 'Aruba'; break;
            case 'AX': return 'Åland Islands'; break;
            case 'AZ': return 'Azerbaijan'; break;
            case 'BA': return 'Bosnia and Herzegovina'; break;
            case 'BB': return 'Barbados'; break;
            case 'BD': return 'Bangladesh'; break;
            case 'BE': return 'Belgium'; break;
            case 'BF': return 'Burkina Faso'; break;
            case 'BG': return 'Bulgaria'; break;
            case 'BH': return 'Bahrain'; break;
            case 'BI': return 'Burundi'; break;
            case 'BJ': return 'Benin'; break;
            case 'BL': return 'Saint Barthélemy'; break;
            case 'BM': return 'Bermuda'; break;
            case 'BN': return 'Brunei'; break;
            case 'BO': return 'Bolivia'; break;
            case 'BQ': return 'British Antarctic Territory'; break;
            case 'BR': return 'Brazil'; break;
            case 'BS': return 'Bahamas'; break;
            case 'BT': return 'Bhutan'; break;
            case 'BV': return 'Bouvet Island'; break;
            case 'BW': return 'Botswana'; break;
            case 'BY': return 'Belarus'; break;
            case 'BZ': return 'Belize'; break;
            case 'CA': return 'Canada'; break;
            case 'CC': return 'Cocos [Keeling] Islands'; break;
            case 'CD': return 'Congo - Kinshasa'; break;
            case 'CF': return 'Central African Republic'; break;
            case 'CG': return 'Congo - Brazzaville'; break;
            case 'CH': return 'Switzerland'; break;
            case 'CI': return 'Côte d’Ivoire'; break;
            case 'CK': return 'Cook Islands'; break;
            case 'CL': return 'Chile'; break;
            case 'CM': return 'Cameroon'; break;
            case 'CN': return 'China'; break;
            case 'CO': return 'Colombia'; break;
            case 'CR': return 'Costa Rica'; break;
            case 'CS': return 'Serbia and Montenegro'; break;
            case 'CT': return 'Canton and Enderbury Islands'; break;
            case 'CU': return 'Cuba'; break;
            case 'CV': return 'Cape Verde'; break;
            case 'CX': return 'Christmas Island'; break;
            case 'CY': return 'Cyprus'; break;
            case 'CZ': return 'Czech Republic'; break;
            case 'DD': return 'East Germany'; break;
            case 'DE': return 'Germany'; break;
            case 'DJ': return 'Djibouti'; break;
            case 'DK': return 'Denmark'; break;
            case 'DM': return 'Dominica'; break;
            case 'DO': return 'Dominican Republic'; break;
            case 'DZ': return 'Algeria'; break;
            case 'EC': return 'Ecuador'; break;
            case 'EE': return 'Estonia'; break;
            case 'EG': return 'Egypt'; break;
            case 'EH': return 'Western Sahara'; break;
            case 'ER': return 'Eritrea'; break;
            case 'ES': return 'Spain'; break;
            case 'ET': return 'Ethiopia'; break;
            case 'FI': return 'Finland'; break;
            case 'FJ': return 'Fiji'; break;
            case 'FK': return 'Falkland Islands'; break;
            case 'FM': return 'Micronesia'; break;
            case 'FO': return 'Faroe Islands'; break;
            case 'FQ': return 'French Southern and Antarctic Territories'; break;
            case 'FR': return 'France'; break;
            case 'FX': return 'Metropolitan France'; break;
            case 'GA': return 'Gabon'; break;
            case 'GB': return 'United Kingdom'; break;
            case 'GD': return 'Grenada'; break;
            case 'GE': return 'Georgia'; break;
            case 'GF': return 'French Guiana'; break;
            case 'GG': return 'Guernsey'; break;
            case 'GH': return 'Ghana'; break;
            case 'GI': return 'Gibraltar'; break;
            case 'GL': return 'Greenland'; break;
            case 'GM': return 'Gambia'; break;
            case 'GN': return 'Guinea'; break;
            case 'GP': return 'Guadeloupe'; break;
            case 'GQ': return 'Equatorial Guinea'; break;
            case 'GR': return 'Greece'; break;
            case 'GS': return 'South Georgia and the South Sandwich Islands'; break;
            case 'GT': return 'Guatemala'; break;
            case 'GU': return 'Guam'; break;
            case 'GW': return 'Guinea-Bissau'; break;
            case 'GY': return 'Guyana'; break;
            case 'HK': return 'Hong Kong SAR China'; break;
            case 'HM': return 'Heard Island and McDonald Islands'; break;
            case 'HN': return 'Honduras'; break;
            case 'HR': return 'Croatia'; break;
            case 'HT': return 'Haiti'; break;
            case 'HU': return 'Hungary'; break;
            case 'ID': return 'Indonesia'; break;
            case 'IE': return 'Ireland'; break;
            case 'IL': return 'Israel'; break;
            case 'IM': return 'Isle of Man'; break;
            case 'IN': return 'India'; break;
            case 'IO': return 'British Indian Ocean Territory'; break;
            case 'IQ': return 'Iraq'; break;
            case 'IR': return 'Iran'; break;
            case 'IS': return 'Iceland'; break;
            case 'IT': return 'Italy'; break;
            case 'JE': return 'Jersey'; break;
            case 'JM': return 'Jamaica'; break;
            case 'JO': return 'Jordan'; break;
            case 'JP': return 'Japan'; break;
            case 'JT': return 'Johnston Island'; break;
            case 'KE': return 'Kenya'; break;
            case 'KG': return 'Kyrgyzstan'; break;
            case 'KH': return 'Cambodia'; break;
            case 'KI': return 'Kiribati'; break;
            case 'KM': return 'Comoros'; break;
            case 'KN': return 'Saint Kitts and Nevis'; break;
            case 'KP': return 'North Korea'; break;
            case 'KR': return 'South Korea'; break;
            case 'KW': return 'Kuwait'; break;
            case 'KY': return 'Cayman Islands'; break;
            case 'KZ': return 'Kazakhstan'; break;
            case 'LA': return 'Laos'; break;
            case 'LB': return 'Lebanon'; break;
            case 'LC': return 'Saint Lucia'; break;
            case 'LI': return 'Liechtenstein'; break;
            case 'LK': return 'Sri Lanka'; break;
            case 'LR': return 'Liberia'; break;
            case 'LS': return 'Lesotho'; break;
            case 'LT': return 'Lithuania'; break;
            case 'LU': return 'Luxembourg'; break;
            case 'LV': return 'Latvia'; break;
            case 'LY': return 'Libya'; break;
            case 'MA': return 'Morocco'; break;
            case 'MC': return 'Monaco'; break;
            case 'MD': return 'Moldova'; break;
            case 'ME': return 'Montenegro'; break;
            case 'MF': return 'Saint Martin'; break;
            case 'MG': return 'Madagascar'; break;
            case 'MH': return 'Marshall Islands'; break;
            case 'MI': return 'Midway Islands'; break;
            case 'MK': return 'Macedonia'; break;
            case 'ML': return 'Mali'; break;
            case 'MM': return 'Myanmar [Burma]'; break;
            case 'MN': return 'Mongolia'; break;
            case 'MO': return 'Macau SAR China'; break;
            case 'MP': return 'Northern Mariana Islands'; break;
            case 'MQ': return 'Martinique'; break;
            case 'MR': return 'Mauritania'; break;
            case 'MS': return 'Montserrat'; break;
            case 'MT': return 'Malta'; break;
            case 'MU': return 'Mauritius'; break;
            case 'MV': return 'Maldives'; break;
            case 'MW': return 'Malawi'; break;
            case 'MX': return 'Mexico'; break;
            case 'MY': return 'Malaysia'; break;
            case 'MZ': return 'Mozambique'; break;
            case 'NA': return 'Namibia'; break;
            case 'NC': return 'New Caledonia'; break;
            case 'NE': return 'Niger'; break;
            case 'NF': return 'Norfolk Island'; break;
            case 'NG': return 'Nigeria'; break;
            case 'NI': return 'Nicaragua'; break;
            case 'NL': return 'Netherlands'; break;
            case 'NO': return 'Norway'; break;
            case 'NP': return 'Nepal'; break;
            case 'NQ': return 'Dronning Maud Land'; break;
            case 'NR': return 'Nauru'; break;
            case 'NT': return 'Neutral Zone'; break;
            case 'NU': return 'Niue'; break;
            case 'NZ': return 'New Zealand'; break;
            case 'OM': return 'Oman'; break;
            case 'PA': return 'Panama'; break;
            case 'PC': return 'Pacific Islands Trust Territory'; break;
            case 'PE': return 'Peru'; break;
            case 'PF': return 'French Polynesia'; break;
            case 'PG': return 'Papua New Guinea'; break;
            case 'PH': return 'Philippines'; break;
            case 'PK': return 'Pakistan'; break;
            case 'PL': return 'Poland'; break;
            case 'PM': return 'Saint Pierre and Miquelon'; break;
            case 'PN': return 'Pitcairn Islands'; break;
            case 'PR': return 'Puerto Rico'; break;
            case 'PS': return 'Palestinian Territories'; break;
            case 'PT': return 'Portugal'; break;
            case 'PU': return 'U.S. Miscellaneous Pacific Islands'; break;
            case 'PW': return 'Palau'; break;
            case 'PY': return 'Paraguay'; break;
            case 'PZ': return 'Panama Canal Zone'; break;
            case 'QA': return 'Qatar'; break;
            case 'RE': return 'Réunion'; break;
            case 'RO': return 'Romania'; break;
            case 'RS': return 'Serbia'; break;
            case 'RU': return 'Russia'; break;
            case 'RW': return 'Rwanda'; break;
            case 'SA': return 'Saudi Arabia'; break;
            case 'SB': return 'Solomon Islands'; break;
            case 'SC': return 'Seychelles'; break;
            case 'SD': return 'Sudan'; break;
            case 'SE': return 'Sweden'; break;
            case 'SG': return 'Singapore'; break;
            case 'SH': return 'Saint Helena'; break;
            case 'SI': return 'Slovenia'; break;
            case 'SJ': return 'Svalbard and Jan Mayen'; break;
            case 'SK': return 'Slovakia'; break;
            case 'SL': return 'Sierra Leone'; break;
            case 'SM': return 'San Marino'; break;
            case 'SN': return 'Senegal'; break;
            case 'SO': return 'Somalia'; break;
            case 'SR': return 'Suriname'; break;
            case 'ST': return 'São Tomé and Príncipe'; break;
            case 'SU': return 'Union of Soviet Socialist Republics'; break;
            case 'SV': return 'El Salvador'; break;
            case 'SY': return 'Syria'; break;
            case 'SZ': return 'Swaziland'; break;
            case 'TC': return 'Turks and Caicos Islands'; break;
            case 'TD': return 'Chad'; break;
            case 'TF': return 'French Southern Territories'; break;
            case 'TG': return 'Togo'; break;
            case 'TH': return 'Thailand'; break;
            case 'TJ': return 'Tajikistan'; break;
            case 'TK': return 'Tokelau'; break;
            case 'TL': return 'Timor-Leste'; break;
            case 'TM': return 'Turkmenistan'; break;
            case 'TN': return 'Tunisia'; break;
            case 'TO': return 'Tonga'; break;
            case 'TR': return 'Turkey'; break;
            case 'TT': return 'Trinidad and Tobago'; break;
            case 'TV': return 'Tuvalu'; break;
            case 'TW': return 'Taiwan'; break;
            case 'TZ': return 'Tanzania'; break;
            case 'UA': return 'Ukraine'; break;
            case 'UG': return 'Uganda'; break;
            case 'UM': return 'U.S. Minor Outlying Islands'; break;
            case 'US': return 'United States'; break;
            case 'UY': return 'Uruguay'; break;
            case 'UZ': return 'Uzbekistan'; break;
            case 'VA': return 'Vatican City'; break;
            case 'VC': return 'Saint Vincent and the Grenadines'; break;
            case 'VD': return 'North Vietnam'; break;
            case 'VE': return 'Venezuela'; break;
            case 'VG': return 'British Virgin Islands'; break;
            case 'VI': return 'U.S. Virgin Islands'; break;
            case 'VN': return 'Vietnam'; break;
            case 'VU': return 'Vanuatu'; break;
            case 'WF': return 'Wallis and Futuna'; break;
            case 'WK': return 'Wake Island'; break;
            case 'WS': return 'Samoa'; break;
            case 'YD': return "People's Democratic Republic of Yemen"; break;
            case 'YE': return 'Yemen'; break;
            case 'YT': return 'Mayotte'; break;
            case 'ZA': return 'South Africa'; break;
            case 'ZM': return 'Zambia'; break;
            case 'ZW': return 'Zimbabwe'; break;
            case 'ZZ': return 'Unknown or Invalid Region'; break;
        }
    }

}
