<?php

namespace AppBundle\Controller;

use AppBundle\Api\CsServerSettings;
use AppBundle\Entity\User;
use AppBundle\Form\Settings;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Class SettingsController
 * @package AppBundle\Controller
 */
class SettingsController extends Controller
{

    /**
     * @Route("/settings", name="app_settings")
     *
     * @param Request $request Request
     *
     * @return Response
     */
    public function settingsAction(Request $request){

        $this->denyAccessUnlessGranted(User::ROLE_SYSTEM_ADMIN);

        /** @var CsServerSettings $apiCsServerSettings */
        $apiCsServerSettings = $this->container->get('cs_api_server_settings');

        $settings = $this->container->get('settings');

        $sections = $apiCsServerSettings->getSettingsSectionsInfo();

        $sectionNames = array();
        
        // Get names of only those sections that contain some public settings.
        foreach ($sections as $section){

            if (is_array($apiCsServerSettings->getSettingsBySection($section['section_id'])) && count($apiCsServerSettings->getSettingsBySection($section['section_id'])) !== 0){
                array_push($sectionNames, $section['name']);
            }

        }

        //$sectionNames = str_replace("_"," ",$sectionNames);
        //var_dump($sectionNames);
        
        $initialData = get_object_vars($settings);

        $form = $this->createForm(Settings::class, $settings, array('apiCsSettings' => $apiCsServerSettings));

        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {

            $toUpdateArray = $this->md_array_diff(get_object_vars($settings), $initialData);

            if(!empty($toUpdateArray)){

                $result = $apiCsServerSettings->updateSettings($toUpdateArray);

                if ($result) {

                    $this->addFlash(
                        'error',
                        'Settings were not saved: '. $result
                    );

                } else {

                    $this->addFlash(
                        'notice',
                        'Your settings were saved!'
                    );

                }

            }

            return $this->redirectToRoute('app_settings');

        }

        return $this->render('app/pages/settings/settings.html.twig', [
            'form' => $form->createView(),
            'sectionNames' => $sectionNames,
        ]);

    }

    /**
     * @param array $array1
     * @param array $array2
     * @param array|null $_
     * @return array
     */
    private function md_array_diff(array $array1, array $array2, array $_ = null) {
        $diff = [];
        $args = array_slice(func_get_args(), 1);

        foreach ($array1 as $key => $value) {
            foreach ($args as $item) {
                if (is_array($item)) {
                    if (array_key_exists($key, $item)) {
                        if (is_array($value) && is_array($item[$key])) {
                            $tmpDiff = $this->md_array_diff($value, $item[$key]);

                            if (!empty($tmpDiff)) {
                                foreach ($tmpDiff as $tmpKey => $tmpValue) {
                                    if (isset($item[$key][$tmpKey])) {
                                        if (is_array($value[$tmpKey]) && is_array($item[$key][$tmpKey])) {
                                            $newDiff = array_diff($value[$tmpKey], $item[$key][$tmpKey]);
                                        } else if ($value[$tmpKey] !== $item[$key][$tmpKey]) {
                                            $newDiff = $value[$tmpKey];
                                        }

                                        if (isset($newDiff)) {
                                            $diff[$key][$tmpKey] = $newDiff;
                                        }
                                    } else {
                                        $diff[$key][$tmpKey] = $tmpDiff;
                                    }
                                }
                            }
                        } else if ($value !== $item[$key]) {
                            $diff[$key] = $value;

                        }
                    } else {
                        $diff[$key] = $value;
                    }
                }
            }
        }

        return $diff;
    }

}
