<?php

namespace AppBundle\Controller;

use AppBundle\Entity\User;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use AppBundle\Form\UserFormType;
use FOS\UserBundle\Model\UserManager;

/**
 * UserController.
 */
class UserController extends Controller
{
    /**
     * Render User page.
     *
     * @Route("users/", name="app_users_management")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function usersAction(Request $request)
    {
        $this->denyAccessUnlessGranted(User::ROLE_SYSTEM_ADMIN);

        /** @var UserManager $userManager */
        $userManager = $this->container->get('fos_user.user_manager');
        $users = $userManager->findUsers();

        return $this->render('app/pages/user/index.html.twig', ['users' => $users]);
    }

    /**
     * @Route("user/add", name="app_user_add")
     *
     * @param Request $request Request
     *
     * @return Response
     */
    public function addUserAction(Request $request)
    {
        /** @var UserManager $userManager */
        $userManager = $this->get('fos_user.user_manager');

        $this->denyAccessUnlessGranted(User::ROLE_SYSTEM_ADMIN);

        $roleHierarchy = $this->container->getParameter('security.role_hierarchy.roles');
        $roles = array_keys($roleHierarchy);

        foreach ($roles as $role) {
            $theRoles[$role] = $role;
        }

        $user = $userManager->createUser();

        $user->setEnabled(true); //checkbox is enabled by default

        $form = $this->createForm(UserFormType::class, $user, array('roles' => $theRoles));
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $user->setPlainPassword($user->getPassword());
            $userManager->updatePassword($user);

            $userManager->updateUser($user);

            $this->addFlash(
                'notice',
                'User was sucessfully created.'
            );

            return $this->redirectToRoute('app_users_management');
        }

        return $this->render('app/pages/user/userEdit.html.twig', [
            'form' => $form->createView(),
        ]);
    }

    /**
     * @Route("user/{userId}", name="app_user_edit")
     *
     * @param Request $request Request
     * @param int     $userId
     *
     * @return Response
     */
    public function editUserAction(Request $request, $userId)
    {
        /** @var UserManager $userManager */
        $userManager = $this->get('fos_user.user_manager');

        $user = $userManager->findUserBy(array('id' => $userId));

        if (!$user) {
            return $this->redirectToRoute('app_users_management');
        }

        $this->denyAccessUnlessGranted(User::ROLE_SYSTEM_ADMIN);

        $roleHierarchy = $this->container->getParameter('security.role_hierarchy.roles');
        $roles = array_keys($roleHierarchy);

        foreach ($roles as $role) {
            $theRoles[$role] = $role;
        }

        $originalPassword = $user->getPassword();


        $form = $this->createForm(UserFormType::class, $user, array('roles' => $theRoles));
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            if ($user->getPassword()) {
                $user->setPlainPassword($user->getPassword());
                $userManager->updatePassword($user);
            } else {
                $user->setPassword($originalPassword);
            }

            $userManager->updateUser($user);

            $this->addFlash(
                'notice',
                'User was sucessfully updated.'
            );

            return $this->redirectToRoute('app_users_management');
        }

        return $this->render('app/pages/user/userEdit.html.twig', [
            'form' => $form->createView(),
        ]);
    }
}
