<?php

use Symfony\Bundle\FrameworkBundle\Client;
use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;
use PHPUnit\DbUnit\TestCaseTrait;


/**
 * Class AdminControllerTest
 * @package App
 * @coverClass \App\Controller\AdminController
 */
class ApiTest extends WebTestCase
{

    use TestCaseTrait;

    /**
     * @var \AppBundle\Api\Device
     */
    private $api;

    // only instantiate pdo once for test clean-up/fixture load
    static private $pdo = null;

    // only instantiate PHPUnit_Extensions_Database_DB_IDatabaseConnection once per test
    private $conn = null;

    /**
     * @return \PHPUnit\DbUnit\Database\DefaultConnection|null
     */
    final public function getConnection()
    {
        if ($this->conn === null) {

            if (self::$pdo == null) {

                $filename = '/opt/vpnportal/customer_server/database/database.db';

                $filename = realpath($filename);

                self::$pdo = new PDO( 'sqlite:' . $filename );

            }

            $this->conn = $this->createDefaultDBConnection(self::$pdo);

        }

        return $this->conn;
    }

    /**
     * @return PHPUnit_Extensions_Database_DataSet_IDataSet
     */
    public function getDataSet()
    {
        return $this->createFlatXMLDataSet(dirname(__FILE__).'/../Data/devices.xml');
    }

    /**
     * Setup Tests
     */
    public function setUp(){

        self::bootKernel();

        $this->api = static::$kernel->getContainer()->get('db_api');

        $this->getConnection()->getConnection()->query('UPDATE "device_table" SET "validated" = 0 WHERE device_id = "3"');

    }

    /**
     * Test devices
     */
    public function testDevices(){

        $deviceCount = $this->api->getCountOfAllDevices();

        $result = $this->api->getAllDevices(0, $deviceCount);

        $this->assertEquals($deviceCount, count($result));

        $resultDevice = $result[0];
        $deviceId = $resultDevice['device_id'];

        $result = $this->api->getDeviceById($deviceId);

        $this->assertEquals($resultDevice, $result);

        //var_dump($result);

        $result = $this->api->getLans($deviceId);

        $this->assertEquals(3, count($result));

        $result = $this->api->getDeviceStats($deviceId);

        $this->assertEquals(1, $result['validated']);

        //var_dump($result);

    }

    public function testNetworks(){

        $networksCount = $this->api->getCountOfAllNetworks();

        $result = $this->api->getAllNetworks(0,$networksCount);

        $this->assertEquals($networksCount, count($result));

        $resultNetwork = $result[0];
        $networkId = $resultNetwork['network_id'];
        $networkName = $resultNetwork['name'];

        $result = $this->api->getNetworkById($networkId);

        $this->assertEquals($result['name'], $networkName);

        /*$result = $this->api->addNetwork('NetworkTest');

        $networksCountAdd = $this->api->getCountOfAllNetworks();

        $this->assertEquals($networksCountAdd, $networksCount + 1);*/

    }

    public function testValidateDevice(){

        $filter['validated'] = '0';

        $devices = $this->api->getAllDevices(0, 10, array(), $filter);

        $deviceId = $devices[0]['device_id'];

        var_dump($deviceId);

        $this->assertEquals(1, count($devices));

        $result = $this->api->validateDevice($deviceId);

        var_dump($result);

        $devices = $this->api->getAllDevices(0, 10, array(), $filter);

        $this->assertEquals(0, count($devices));



        //var_dump($devices);

    }


}